#!/bin/env perl

# usage : perl sm_FASTQ_qual.pl <file.fastq>
# 26/09/2015 
# Sarah Maman
# Copyright (C) 2015 INRA
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


use strict;
use warnings;
defined $ENV{'MY_GALAXY_DIR'} || die "MY_GALAXY_DIR environment variable not defined";
use lib "$ENV{'MY_GALAXY_DIR'}";
use GalaxyPath;
use File::Basename;
use Getopt::Long;
use POSIX; 
use File::Basename;
use 5.010;  #for filesize
 
#************************************************************************************
#******                        Variables Declaration                             ****
#************************************************************************************ 
my $cfg = GalaxyPath->new( -file => $ENV{"GALAXY_CONFIG_FILE"} );
my $myworkspace_path = $cfg->my_path( 'workPath', 'MYWORKSPACE' );
$myworkspace_path=~s/\/$//; # remove trailing '/' in the path
my $scripts_fosmides= $cfg->my_path( 'workPath', 'FOSMIDES_PATH_MODULE1' );
my $SPADES = $cfg->my_path( 'toolsPath', 'SPADES390_PATH' );
my $CROSS_MATCH = $cfg->my_path( 'toolsPath', 'CROSS_MATCH_PATH' );
my $INFOSEQ = $cfg->my_path( 'toolsPath', 'INFOSEQ_PATH' );
my $FASTASELECT = $cfg->my_path( 'toolsPath', 'FASTASELECT_PATH' );
my $FASTALENGTH = $cfg->my_path( 'toolsPath', 'FASTALENGTH_PATH' );
my $FASTQ2FASTA = $cfg->my_path( 'toolsPath', 'FASTQ2FASTA_PATH' );
my $datatypeusefilelist; 
my $datatypeinputFastqGz;  #to replace datatypeinputFastqGz MiSeq and Proton
my $FosmidLength;
my $Minimallengthcontig;
my $Minimaldepthcontig;
my $vectorseq;
my $Coverage;
my $output;
my $nbFiles;
my $nbSamples;
my $cmd1;
my $cmd2;
my $lin;
my $url;
my $url2;
my $url3;
my $url4;
my $file100X;
my $outputzip;


Getopt::Long::Configure( 'no_ignorecase', 'bundling' );
GetOptions (
    'input=s' => \$datatypeinputFastqGz,
    'FosmidLength=i' => \$FosmidLength,
    'Minimallengthcontig=i' => \$Minimallengthcontig,
    'Minimaldepthcontig=i' => \$Minimaldepthcontig,
    'vectorseq=s' => \$vectorseq, 
    'Coverage=i' => \$Coverage,
    'outputzip=s' => \$outputzip,
    'output=s' => \$output
) or die "Usage: Error in command line arguments\n";
     
 	     
     
#************************************************************************************
#******                         Workspace                                        ****
#************************************************************************************        

#Unic number of Galaxy Output in order to create, if necessary, a directory
#for tests in /work/galaxy-dev/workspace and an unic subdirectory with Galaxy output number
#to follow your tests becaus temporary datasets are deleted by Galaxy in job_working_directory
my ($nb) = ($output=~/galaxy_dataset_(\d+)\.\S+$/);

#My workspace directory created by this script. Check permissions :
#write permission and execution for this output directory.
my $cmdini = "(cd $myworkspace_path/; mkdir $nb/; chmod -R 777 $nb/; cd $nb/;) >& ./cd.log 2>&1";
system $cmdini;
my $dirresults= $myworkspace_path."/".$nb;                       # For barecode error
my $dir = "$ENV{MY_GALAXY_DIR}/my_workspace/";



    
#************************************************************************************
#******                         General methods                                  ****
#************************************************************************************        
 
sub check {
    if ( WIFEXITED($?) ) {    # true for a normal end
            my $status = WEXITSTATUS($?);    # exit state
        print STDOUT "etat du process : $status \n";
        }
}

#************************************************************************************
#******                         Data informations                                ****
#************************************************************************************ 
my $file;

#Recover input data files in your workspace (if debug) or in job_working_directory (if production)
sub dataRecup {
        my $cmdcp="(cd $dirresults/; mkdir DATA/; chmod -R 777 DATA/; cp $datatypeinputFastqGz $dirresults/DATA.zip; unzip $dirresults/DATA.zip -d $dirresults/DATA/; mv $dirresults/DATA/*/*.gz $dirresults/DATA/.) >& ./cpfosmides.log 2>&1"; 
        system $cmdcp; 
        print STDOUT "Data preparation : $cmdcp \n\n";
}

#Determine whether input data files are paired or single fastq files
my $paired;
sub searchPairedFiles {
    no warnings 'numeric';
    my $tmp = `ls $dirresults/DATA/*.f* | grep "R2.f" | wc -l;`;
    if($tmp == "0"){$paired="Single data";}else{$paired="Paired data";}
    print STDOUT "$tmp ******** paired : $paired \n";
    return $paired;
}
    
#For paired data input files, count the number of files (R2 only)    
my $ps;
sub pairedFiles {
    no warnings 'numeric';
    my $tmp = `ls $dirresults/DATA/*.f* | grep "R2.f" | wc -l;`;
    if ($tmp == "0") {$ps="Single data";}else{$ps="Paired data";}
    return $ps;
}


#In your input zipped directory, count the total number of fastq files
my $nbfilestot;
sub nbfiles {
    #Total number of files in your zip directory
    $nbfilestot = `ls $dirresults/DATA/*.f*q | wc -l;`;
    return $nbfilestot;    
}

#Files sizes
sub infoSize {
    my ($file) = @_;
    my $size = -s $file;
    return $size;
}    
#Files names
sub infoFiles {
     my ($file) = @_;
     my $names = basename($file); 
     $names =~s/.gz//;
     return $names;     
}

#Count number of sequencies in each fastq file
#zcat: print gzipped file (if the file is not zipped, just use cat)
#paste - - - -: print four consecutive lines in one row (tab delimited)
#cut -f2: print only the second column (after paste this is the second line of the fastq-format, meaning the sequence)
#wc -c: count the characters
#tr -d '\n'   --> This will also count newline characters, remove those first.
sub seqCount{
      my ($file) = @_;
      
    if ($file =~ m/gz/) {
             my $countseq=`gunzip -c $file > $dirresults/DATA/fileSeqCount.fastq; $FASTQ2FASTA  < $dirresults/DATA/fileSeqCount.fastq > $dirresults/DATA/fileSeqCount.fasta ; $FASTALENGTH $dirresults/DATA/fileSeqCount.fasta | wc -l; rm -rf $dirresults/DATA/fileSeqCount.fasta;`;
             print STDOUT "seq count file :  gunzip -c $file > $dirresults/DATA/fileSeqCount.fastq; $FASTQ2FASTA  < $dirresults/DATA/fileSeqCount.fastq > $dirresults/DATA/fileSeqCount.fasta ; $FASTALENGTH $dirresults/DATA/fileSeqCount.fasta | wc -l; rm -rf $dirresults/DATA/fileSeqCount.fasta;";
             return $countseq;
         }else{
             my $countseq=`$FASTQ2FASTA  < $file > $dirresults/DATA/fileSeqCount.fasta ; $FASTALENGTH $dirresults/DATA/fileSeqCount.fasta | wc -l; rm -rf $dirresults/DATA/fileSeqCount.fasta;`;       
             print STDOUT "seq count file :  $FASTQ2FASTA  < $file > $dirresults/DATA/fileSeqCount.fasta ; $FASTALENGTH $dirresults/DATA/fileSeqCount.fasta | wc -l; rm -rf $dirresults/DATA/fileSeqCount.fasta;";
             return $countseq;
             }
      
}    

#Count number of bases in each fastq file
sub seqCountBases{
      my ($file) = @_;
      my $countATGC=`cat $file | paste - - - - | cut -f 2 | tr -d '\n' | wc -c;`;
      return $countATGC;
}    


#Check if your files are fastq files or not
sub checkFatsqFile{
    my ($file) = @_;
    my $checkfastqfile;
    if (seqCount($file) == "0"){$checkfastqfile ="Error : Your file is not a fastq format";}else{$checkfastqfile="Your file is a fastq format";}
    return $checkfastqfile;
}


#Samples count  $dirresults/DATA/*.f*.gz
sub sampleCount {
         my ($file) = @_;
        $nbSamples = `ls $file | wc -l;`;
        return $nbSamples ;
}
    
#Some informations available in STDOUT (i icone in Galaxy dataset) for biologist users
print STDOUT "Your working directory on Galaxy workbench : $dirresults\n\n";


#Total number of files available in the zip directory gievd by Galaxy user
my $totNbFiles; 
sub FilesNumber {
       $totNbFiles = `ls $dirresults/DATA/*.f*q | wc -l;`;
       print "Total files number in your zip directory : $totNbFiles";
       return $totNbFiles;
}


#************************************************************************************
#******      MAIN                                                            ******
#************************************************************************************

#Recovory of inputs files
dataRecup();

#Determine if data input files are single data or paired data
#in order to adapt bioinformatics treatments
my $info=searchPairedFiles();

#If input data are single one, datatype are "Proton"
#else, if input data are paired fastq files, so datatype is "MiSeq"
if ($info eq "Single data"){$datatypeusefilelist="Proton";}else{$datatypeusefilelist="MiSeq";}

#Some checks points
check();
    
#Count for total number of samples is different if datatype is MiSeq or if datatype is Proton    
my $sampleCountFiles;    
if ($datatypeusefilelist eq "MiSeq"){
        $sampleCountFiles = (sampleCount("$dirresults/DATA/*.f*.gz"))*0.5;  #R1 and R2 = 1 sample (paired reads)
    }else{
        $sampleCountFiles = sampleCount("$dirresults/DATA/*.f*.gz");
    }    

	
#************************************************************************************
#******                         filtre                                           ****
#************************************************************************************ 
#spades : -t est le nb de threads 
my $j = 1;
my $nbreads;
my $nbreadsProton;
my $meanreads2;
my $meanreads2Proton;
		
if ($datatypeusefilelist eq "MiSeq"){            
            print STDOUT "MiSeq files \n\n";
            my $pwd="$dirresults/DATA/"; # récupère le répertoire courant
            chomp $pwd;
            chdir("$dirresults/DATA/");
            `chmod -R 777 $dirresults/`;
            my @R1 = sort {$a cmp $b } glob("*1.f*"); # les noms des r1, triés   
            my @R2 = sort {$a cmp $b } glob("*2.f*"); # les noms des r2, triés aussi de la même façon  
            $#R1 == $#R2 || die "not same number of read r1 and r2 : $#R1/$#R2"; # un contrôle
            for (my $i=0;$i<=$#R1;$i++) {
                 my ($r1,$r2)=($R1[$i],$R2[$i]);
                   `chmod 777 $r1; chmod 777 $r2`;
                 print STDOUT "Read 1 : $r1 \nRead 2 : $r2 \n\n";              
                 my $cmdfiltrefastq="(cd $dirresults/DATA/; gunzip $r1;) >& ./cpunzip1.log 2>&1"; 
                 system $cmdfiltrefastq; 

            check();
	       
	        my $cmdfiltrefastq2="(cd $dirresults/DATA/; gunzip $r2;) >& ./cpunzip2.log 2>&1"; 
                             system $cmdfiltrefastq2; 

            check();

                 print STDOUT "Uncompress :  gunzip $r1; gunzip $r2;\n\n";
                 my $fastq1=$r1;
                 $fastq1=~s/.gz//; print STDOUT "fastq R1 : $fastq1\n";   
                 my $fastq2=$r2;
                 $fastq2=~s/.gz//; print STDOUT "fastq R2 : $fastq2\n\n";
                 
                 # Read from a pipe
                open(INFOSEQ,"$INFOSEQ -auto -only -length $fastq1 2> /dev/null |") || die "problem with infoseq : PLEASE CHECK YOUR FILE FORMAT";

                # continuous flux
                my ($sum,$count,$meanreads)=(0,-1,0);
                while (my $line=<INFOSEQ>){
                  $count++;
                  # skip header
                  next if($count==0); 
                  chomp $line;
                  $sum+=$line;
                }
                if($count>0){
                  $meanreads=$sum/$count;
                }
                close(INFOSEQ);

                 # Miseq Illumina: paired end reads
                 $nbreads = (ceil(($Coverage*$FosmidLength)/($meanreads*2)))*4; #*4 car blocs de 4 lignes dans fastq
                 print STDOUT "Number of selected reads : $nbreads = ceil((100*$FosmidLength)/($meanreads*2)); \n\n";  
                 $meanreads2 = $meanreads;
        
                  my $read1 = "100X_".$j."_R1.fastq" ;
                  my $read2 = "100X_".$j."_R2.fastq";
                 `head -n $nbreads $fastq1 > $read1 ;`;   
                 `head -n $nbreads $fastq2 > $read2 ;`;  
                  print STDOUT "Filter: head -n $nbreads *1.f* > $read1\nhead -n $nbreads *2.f* > $read2\n\n";      
                  print STDOUT "Data preparation 100 X filter on $read1 and $read2 : meanreads :$meanreads; insert size : $FosmidLength; Number of paired reads: $nbreads \n\n";
                
                  # Loader then run spades on MiSeq data
                  $cmd1 = "(module load bioinfo/SPAdes-3.1.1; $SPADES --pe1-1 $read1 --pe1-2 $read2 -t 8 --careful -o $dirresults/DATA/echantillon_$j) >& ./out_spades_miseq.log 2>&1";       
                  system $cmd1;
                 
                 check();
                  
                  #Compress fastq files for following treatments
                  `gzip $fastq1; gzip $fastq2;`;
                  print STDOUT "Loader and MiSeq spades runs command lines : $cmd1; \n\nCompress fastq files for following treatments: `gzip $fastq1; gzip $fastq2;`\n\n";                
                  print STDOUT "ls $dirresults/echantillon_$j/spades_dir \n\n";
                  $j++;
            }
            chdir($pwd);                                    
}
else{
                 # Recover current directory    
                 my $pwd2="$dirresults/DATA/"; 
                 chomp $pwd2;
                 chdir("$dirresults/DATA/");                                        
                `mkdir $dirresults/DATA/100X/; chmod 777 $dirresults/DATA/100X/;`;
                 my @files = glob("$dirresults/DATA/*.f*");
                    foreach my $file (@files) {
                        print STDOUT "Proton file : $file\n\n"; 
                        `chmod 777 $file;`;                                                  
                         my $cmdgunzip="(cd $dirresults/DATA/; gunzip $file;) >& ./cpunzip2.log 2>&1"; 
                         system $cmdgunzip; 
                         print STDOUT "Uncompress : $cmdgunzip\n\n";
                         my $file1=$file;
                         $file1=~s/.gz$//; print STDOUT "Fastq file : $file1\n\n";   
                                  
                        #Read from a piped command
                        open(INFOSEQ,"$INFOSEQ -auto -only -length $file1 |") || die "problem with infoseq : PLEASE CHECK YOUR FILE FORMAT";
                          
                        # Read in continuus flux
                        my ($sum,$count,$meanreads)=(0,-1,0);
                        while (my $line=<INFOSEQ>){
                          $count++;
                          next if($count==0); # skip header
                          chomp $line;
                          $sum+=$line;
                        }
                        if($count>0){
                          $meanreads=$sum/$count;
                          print STDOUT "Mean reads : $meanreads\n\n";
                        }
                        close(INFOSEQ);
                       
                        # Proton: single end reads
                        $nbreadsProton = (ceil(($Coverage*$FosmidLength)/$meanreads))*4;  
                        print STDOUT "Proton single end reads information: $nbreadsProton = ceil((100*$FosmidLength)/$meanreads)\n\n";   
                        $meanreads2Proton = $meanreads;
                         
                        $file100X = basename($file); 
                        $file100X =~s/.gz//;
                        
                        print STDOUT "Basename : $file100X\n\n";                        
                        
                       `head -n $nbreadsProton $file1  > $dirresults/DATA/100X/$file100X `;                 
                        print STDOUT "Data preparation with 100X filter : meanreads :$meanreads; insert size : $FosmidLength; Number of reads: $nbreadsProton\n\n";
                                                
                    #Create a directory with the sample name in order to save sapes output files by sample
                    my $rep = $file100X; 
                    $rep =~s/.fq//;
                    $rep =~s/.fastq//;
                    `mkdir $dirresults/DATA/SPADES_$rep/; chmod 777 $dirresults/DATA/SPADES_$rep/;`;
                    #Loader and run spades on ion torrent data (proton)
                    $cmd2 = "(module load bioinfo/SPAdes-3.1.1; $SPADES --iontorrent --s1 $dirresults/DATA/100X/$file100X -t 2 --careful -o $dirresults/DATA/SPADES_$rep/;) >& ./out_spades_proton.log 2>&1";
                    system $cmd2;
                    print STDOUT "Loader and run spades on ion torrent data (proton) command lines : $cmd2\n\n";
                    #recompresser les fastq pour la suite des traitements
                    `gzip $dirresults/DATA/100X/$file100X ;`;       
                    print STDOUT "Compress files : gzip $dirresults/DATA/100X/$file100X\n\n";
                    $j++;
            }#fin du for
            chdir($pwd2);   
}


#************************************************************************************
#******  Check process                                                         ******
#************************************************************************************
check();


#************************************************************************************
#******                           CROSS MATCH                                  ******
#************************************************************************************


if ($datatypeusefilelist eq "MiSeq"){
        my $n = ($nbSamples*0.5);
        for (my $i = 1; $i <= $n ; $i++) {
        #Recovery of specifics scripts in your workspace directory
        `ln -s $scripts_fosmides/*.pl $dirresults/DATA/echantillon_$i/.;`;
        `ln -s $scripts_fosmides/process_fosmid_overlap_detection.py  $dirresults/DATA/echantillon_$i/process_fosmid_overlap_detection.py ;`;

        my $cmdCROSS_MATCH = "(chmod -R 777 $dirresults/DATA/echantillon_$i/; cd $dirresults/DATA/echantillon_$i/; $CROSS_MATCH $dirresults/DATA/echantillon_$i/contigs.fasta  $vectorseq  -minmatch 9 -minscore 30 -screen > $dirresults/DATA/echantillon_$i/crossmatch-contigs.fasta) >& ./crossmatch.log 2>&1";
        system $cmdCROSS_MATCH;
        print STDOUT "Cross_match : $cmdCROSS_MATCH;\n\n"; 
         }
}else{
        my @files = glob("$dirresults/DATA/*.f*");
        foreach my $file (@files) {
                $file100X = basename($file); 
                $file100X =~s/.gz//;
                my $rep = $file100X; 
                $rep =~s/.fq//;
                $rep =~s/.fastq//;
                print STDOUT "Sample : $file100X\n\n";
                #Recover scripts  in workspace directory
                `ln -s $scripts_fosmides/*.pl $dirresults/DATA/SPADES_$rep/.;`;
                `ln -s $scripts_fosmides/process_fosmid_overlap_detection.py  $dirresults/DATA/SPADES_$rep/process_fosmid_overlap_detection.py ;`;
                #Loader and run spades on ion torrent data (proton)
                my $cmdCROSS_MATCH2 = "(chmod -R 777 $dirresults/DATA/SPADES_$rep/; cd $dirresults/DATA/SPADES_$rep/; $CROSS_MATCH $dirresults/DATA/SPADES_$rep/contigs.fasta  $vectorseq  -minmatch 9 -minscore 30 -screen > $dirresults/DATA/SPADES_$rep/crossmatch-contigs.fasta) >& ./crossmatch.log 2>&1";
                system $cmdCROSS_MATCH2;
                print STDOUT "Cross_match SPADES_$rep : $cmdCROSS_MATCH2\n\n"; 
          }        
}


#************************************************************************************
#******  Check process                                                         ******
#************************************************************************************
check();
#************************************************************************************
#****** RECUPERER LES CONTIGS LES PLUS LONGS  ******
#************************************************************************************            
if ($datatypeusefilelist eq "MiSeq"){
      my $n =($nbSamples *0.5 );
      for (my $i = 1; $i <= $n; $i++) {
                my $cmdstats = "(cd $dirresults/DATA/echantillon_$i/; ) >& ./cd.log 2>&1;";
                system $cmdstats;
                `perl -lne 'if(\$_=~/^>NODE/){(\$ln,\$cov)=\$_=~/_length_(\\d+)_cov_(\\d+\.*\\d*)/; print "length: \$ln ; cov: \$cov"}' < $dirresults/DATA/echantillon_$i/scaffolds.fasta > $dirresults/DATA/echantillon_$i/FILTRE_$i.txt;`;
                my $cmdinfo1 = "($INFOSEQ -auto -sequence $dirresults/DATA/echantillon_$i/scaffolds.fasta -only -length -outfile $dirresults/DATA/echantillon_$i/read_size_$i.txt;  ) 2> log1 ;";   #scaffolds.fasta est genere a partir des fichier 100X 
                system $cmdinfo1;
                my $cmdinfo2= "($INFOSEQ -auto -only -length -name $dirresults/DATA/echantillon_$i/scaffolds.fasta |sort -k 2 -n -r | head -n1 | awk '{print \$1;}' > $dirresults/DATA/echantillon_$i/id_file.txt)  2> log2 ;";
                system $cmdinfo2;
                # extract the sequence(s) by sequence id
                `$FASTASELECT $dirresults/DATA/echantillon_$i/scaffolds.fasta $dirresults/DATA/echantillon_$i/id_file.txt > $dirresults/DATA/echantillon_$i/longest_contig_Miseq.fasta  `;
                print STDOUT "Infoseq and fasta select\n\n";
       }
}else{

 my @files = glob("$dirresults/DATA/*.f*");
        foreach my $file (@files) {
                $file100X = basename($file); 
                $file100X =~s/.gz//;
                print STDOUT "Sample : $file100X\n\n";
                my $rep = $file100X; 
                $rep =~s/.fq//;
                $rep =~s/.fastq//;
                my $cmdinfo3 ="($INFOSEQ -auto -sequence $dirresults/DATA/SPADES_$rep/scaffolds.fasta -only -length -outfile $dirresults/DATA/SPADES_$rep/read_size_$file100X.txt;) 2> log3 ;";   #scaffolds.fasta est genere a partir des fichier 100X 
                system $cmdinfo3;
                my $cmdinfo4="($INFOSEQ -auto -only -length -name $dirresults/DATA/SPADES_$rep/scaffolds.fasta |sort -k 2 -n -r | head -n1 | awk '{print \$1;}' > $dirresults/DATA/SPADES_$rep/id_file.txt) 2> log4 ;";
                system $cmdinfo4;
                # extract the sequence(s) by sequence id
                `$FASTASELECT $dirresults/DATA/SPADES_$rep/scaffolds.fasta $dirresults/DATA/SPADES_$rep/id_file.txt > $dirresults/DATA/SPADES_$rep/longest_contig_Proton.fasta`;                    
                print STDOUT "Infoseq and fasta select\n\n";
                }  
}       


#************************************************************************************
#******  Check process                                                         ******
#************************************************************************************
check();

#************************************************************************************
#******                         Filter contigs : min len and min cov           ******
#************************************************************************************
 #Recover current directory
 my $pwd="$dirresults/DATA/";
 chomp $pwd;                
 chdir("$dirresults/DATA/"); 
 my @R1 = sort {$a cmp $b } glob("*1.f*"); # les noms des r1, triés       
 my @R2 = sort {$a cmp $b } glob("*2.f*"); # les noms des r2, triés aussi de la même façon  
 if ($datatypeusefilelist eq "MiSeq"){$#R1 == $#R2 || die "not same number of read r1 and r2 : $#R1/$#R2"; }# un contrôle          

if ($datatypeusefilelist eq "MiSeq"){  
        my $n =($nbSamples *0.5); 
        for (my $i = 1; $i <= $n; $i++) {
                    #extract all subsequence from crossmatch screen to generate contigs fasta screen cleaned
                    #cp /usr/local/bioinfo/src/python/current/bin/python binary/.
                    #my $cmdfilterfasta="(cd $dirresults/DATA/echantillon_$i/; ln -s $scripts_fosmides/extract_all_subsequence_from_crossmatch_screen_noqual.py .; /usr/local/bioinfo/src/python/current/bin/python extract_all_subsequence_from_crossmatch_screen_noqual.py contigs.fasta.screen;) >& ./extract.log 2>&1;";
                    my $cmdfilterfasta="(cd $dirresults/DATA/echantillon_$i/; ln -s $scripts_fosmides/extract_all_subsequence_from_crossmatch_screen_noqual.py .; $scripts_fosmides/binary/python extract_all_subsequence_from_crossmatch_screen_noqual.py contigs.fasta.screen;) >& ./extract.log 2>&1;";
                    system $cmdfilterfasta;
                    #recuperation of vectors stats in a VECTOR.log file
                    `cp ./extract.log $dirresults/DATA/echantillon_$i/VECTOR.log`;
                    print STDOUT "Filter fasta: $cmdfilterfasta\n\n";
                    
                    #From contigs fasta screen clean, generate un statistiques file with filtered contigs
                    my $cmdfilter = "(cd $dirresults/DATA/echantillon_$i/; $FASTALENGTH contigs.fasta.screen.clean |sort -nr > $dirresults/DATA/echantillon_$i/filtred_contigs.txt;) >& ./filter_contigs.log 2>&1;";
                    system $cmdfilter;
                    #Info pour les biologistes
                    print STDOUT "Filter contigs : $cmdfilter\n\n";
                     
                    #print these stats in output files
                    open (ALLCONT,"$dirresults/DATA/echantillon_$i/filtred_contigs.txt")or die "Impossible to open $dirresults/DATA/echantillon_$i/filtred_contigs.txt";
	                open (AVFASTA,">$dirresults/DATA/echantillon_$i/stats.txt") or die "Impossible to open $dirresults/DATA/echantillon_$i/stats.txt"; 
                    open (SELECTED,">$dirresults/DATA/echantillon_$i/selected.txt") or die "Impossible to open $dirresults/DATA/echantillon_$i/selected.txt"; 

                    #Legend :  Name of assembled contig - Length - Coverage
	                print AVFASTA  "Name of assembled contig \t Length \t Coverage\n";

	                while(defined($lin = <ALLCONT>)) {
		                chomp($lin);
		                my @lin=split(/ /,$lin);
		                my ($contigName)=$lin[1];
                        my ($d) = $lin =~ /cov_(\d+\.*\d*)/; #la coverage est un nombre décimal
                        my ($l) = $lin =~ /length_(\d+)_cov/;
                  		print AVFASTA $contigName."\t".$l."\t".$d."\n";
                        #Only stats for min depth and min lenght contig given by the galaxy user
                  		if (($d > $Minimaldepthcontig ) && ($l > $Minimallengthcontig)){print SELECTED $contigName."\n" ;}
	                } 
	
	                #generate selected fasta
                    # cp /usr/local/bioinfo/Scripts/bin/fasta_extract.pl binary/.
	                #my $filterFASTA=("perl /usr/local/bioinfo/Scripts/bin/fasta_extract.pl < $dirresults/DATA/echantillon_$i/contigs.fasta.screen.clean $dirresults/DATA/echantillon_$i/selected.txt > $dirresults/DATA/echantillon_$i/selected.fasta; ");
                    my $filterFASTA=("perl $scripts_fosmides/binary/fasta_extract.pl < $dirresults/DATA/echantillon_$i/contigs.fasta.screen.clean $dirresults/DATA/echantillon_$i/selected.txt > $dirresults/DATA/echantillon_$i/selected.fasta; ");
	                system $filterFASTA; 
	                print STDOUT "Filter FASTA : $filterFASTA\n\n";
	
	                #Close opened files
	                close (ALLCONT);				
	                close (AVFASTA);
	                close (SELECTED);
                   
                    #Inform biologists users
                    print STDOUT "Statistics files : $dirresults/DATA/echantillon_$i/stats.txt  and  $dirresults/DATA/echantillon_$i/selected.txt;\n\n";           
       }
}else{
        my @files = glob("$dirresults/DATA/*.f*");
        foreach my $file (@files) {
                $file100X = basename($file); 
                $file100X =~s/.gz//;
                print STDOUT "Sample : $file100X\n\n";
                my $rep = $file100X; 
                $rep =~s/.fq//;
                $rep =~s/.fastq//;
                #my $cmdfilterfasta2="(cd $dirresults/DATA/SPADES_$rep/; ln -s $scripts_fosmides/extract_all_subsequence_from_crossmatch_screen_noqual.py .; /usr/local/bioinfo/src/python/current/bin/python extract_all_subsequence_from_crossmatch_screen_noqual.py contigs.fasta.screen;) >& ./extract.log 2>&1; ";
                my $cmdfilterfasta2="(cd $dirresults/DATA/SPADES_$rep/; ln -s $scripts_fosmides/extract_all_subsequence_from_crossmatch_screen_noqual.py .; $scripts_fosmides/binary/python extract_all_subsequence_from_crossmatch_screen_noqual.py contigs.fasta.screen;) >& ./extract.log 2>&1; ";
                system $cmdfilterfasta2;
                #Recover vectors stats in a VECTOR.log file
                `cp ./extract.log $dirresults/DATA/SPADES_$rep/VECTOR.log`;
                print STDOUT "Filter fasta: $cmdfilterfasta2\n\n";
                                
                #Loader and run spades on ion torrent data (proton)
                my $cmdfilter2 = "(cd $dirresults/DATA/SPADES_$rep; $FASTALENGTH contigs.fasta.screen.clean |sort -nr > $dirresults/DATA/SPADES_$rep/filtred_contigs.txt;) >& ./filter_contigs.log 2>&1;";
                system $cmdfilter2;
                #Inform biologists users
                print STDOUT "Filter contigs : $cmdfilter2\n\n";
                
                open (ALLCONT,"$dirresults/DATA/SPADES_$rep/filtred_contigs.txt") or die "Impossible to open $dirresults/DATA/SPADES_$rep/filtred_contigs.txt";
	            open (AVFASTA,">$dirresults/DATA/SPADES_$rep/stats.txt") or die "Impossible to open $dirresults/DATA/SPADES_$rep/stats.txt";  
	            open (SELECTED,">$dirresults/DATA/SPADES_$rep/selected.txt") or die "Impossible to open $dirresults/DATA/SPADES_$rep/selected.txt";  
	            
                #Legend :  Name of assembled contig - Length - Coverage
	            print AVFASTA  "Name of assembled contig \t Length \t Coverage\n";
                
	            while(defined($lin = <ALLCONT>)) {
		            chomp($lin);
                  	my @lin=split(/ /,$lin);
		            my ($contigName)=$lin[1];
                    my ($d) = $lin =~ /cov_(\d+)./; 
                    my ($l) = $lin =~ /length_(\d+)_cov/; 
                    print AVFASTA  $contigName."\t".$l."\t".$d."\n"; 
                    #Only stats for min depth and min lenght contig given by the galaxy user 
                    if (($d > $Minimaldepthcontig ) && ($l > $Minimallengthcontig)){print SELECTED $contigName."\n" ;}                  
                    } 
                    
                    #generate selected fasta
	                #my $filterFASTA=("perl /usr/local/bioinfo/Scripts/bin/fasta_extract.pl < $dirresults/DATA/SPADES_$rep/contigs.fasta.screen.clean $dirresults/DATA/SPADES_$rep/selected.txt > $dirresults/DATA/SPADES_$rep/selected.fasta; ");
	                my $filterFASTA=("perl $scripts_fosmides/binary/fasta_extract.pl < $dirresults/DATA/SPADES_$rep/contigs.fasta.screen.clean $dirresults/DATA/SPADES_$rep/selected.txt > $dirresults/DATA/SPADES_$rep/selected.fasta; ");
                    system $filterFASTA; 
	                print STDOUT "Filter FASTA : $filterFASTA\n\n";
	                
	                #Inform biologists
                    print STDOUT "Statistics files : $dirresults/DATA/SPADES_$rep/stats.txt  and  $dirresults/DATA/SPADES_$rep/selected.txt;\n\n";           
                	
                #Close opened files
	            close (ALLCONT);				
	            close (AVFASTA);
	            close (SELECTED);        
	} 
}
                
#************************************************************************************
#******  Check process                                                         ******
#************************************************************************************
check();

              
#************************************************************************************
#******                           CreateVMAndVNFiles-dec2013.pl                ******
#************************************************************************************
 chomp $pwd;   
 chdir("$dirresults/DATA/");   

if ($datatypeusefilelist eq "MiSeq"){         
            $j=1;
            my $n = ($nbSamples *0.5);
            for (my $i=1;$i<=$n;$i++) {
                       my ($r1,$r2)=($R1[$i],$R2[$i]);   
                       #file name example : 54J23_TCGAAG_L001_R1.fastq.gz    
                       my ($clonename)=$r1 =~/^([^_]+)_/;                                                           
                       #Run second step of treatment
                       my $cmd3 = "(cd $dirresults/DATA/echantillon_$j/; perl sm_createVNVM.pl $dirresults/DATA/echantillon_$j/filtred_contigs.txt $dirresults/DATA/echantillon_$j/contigs.fasta.screen $clonename $dirresults/DATA/echantillon_$j/) >& ./clone3.log 2>&1;";
                       system $cmd3;
                       #Inform biologists users
                       print STDOUT "Create VN and VM files : $cmd3\n\n";
                       $j++;
       }
       chdir($pwd);
}else{
        #Loader and run spades on ion torrent data (proton)
         my @files = glob("$dirresults/DATA/*.f*");
         foreach my $file (@files) {
                $file100X = basename($file); 
                $file100X =~s/.gz//;
                print STDOUT "Sample : $file100X\n\n";
                my $rep = $file100X; 
                $rep =~s/.fq//;
                $rep =~s/.fastq//;
                my $cmd3 = "(cd $dirresults/DATA/SPADES_$rep/; perl sm_createVNVM.pl $dirresults/DATA/SPADES_$rep/filtred_contigs.txt $dirresults/DATA/SPADES_$rep/contigs.fasta.screen SPADES_$rep $dirresults/DATA/SPADES_$rep/; cp $dirresults/DATA/SPADES_$rep/filtred_contigs.txt $dirresults/DATA/filtred_contigs.txt;) >& ./clone4.log 2>&1;";                
                system $cmd3;
                #Info pour les biologistes
                print STDOUT "Create VN and VM files : $cmd3\n\n";
                }         
}

#************************************************************************************
#******  Check process                                                         ******
#************************************************************************************
check();

#************************************************************************************
#******                                       REPORT HTML                      ******
#************************************************************************************
                            
#Creation of HTML file
open (HTML, ">", "results.html") or die "Cannot create results.html";

#Green color : A9F5A9
print HTML "<!DOCTYPE html>
<html>
<head>
<style>
table {
    width:100%;
}
table, th, td {
    border: 1px solid black;
    border-collapse: collapse;
}
th, td {
    padding: 5px;
    text-align: left;
}
table#t01 tr:nth-child(even) {
    background-color: #eee;
}
table#t01 tr:nth-child(odd) {
   background-color:#fff;
}
table#t01 th	{
    background-color: #A9F5A9;  
    color: white;
}

table#t02 tr:nth-child(even) {
    background-color: #eee;
}
table#t02 tr:nth-child(odd) {
   background-color:#fff;
}
table#t02 th	{
    background-color: #A9F5A9;  
    color: white;
}
</style>
</head>
<body>
";


print HTML "<strong><center><h1 style='font-family:verdana;'>PIPELINE FOR CONTIG ANNOTATION</h></center></strong><br>";
print HTML "<strong><center><h1 style='font-family:verdana;'>MODULE 1 : Contig preparation and general statistics</h></center></strong><br>";
print HTML "<div style='background-color:#A9F5A9; color:white; padding:20px;'>";
print HTML "<center><strong>RESULTS : CONTIGS FINALLY KEPTS</strong></center><br>";
print HTML "</div>";

my $nbfilesAff = nbfiles();
print HTML "Your ZIP directory contains <strong> $nbfilesAff valid files </strong>.";

my $nom;
my $seq;
my $pairedsingle;
my $sizecount;
my $fastqformat;
my $ATGC;

print HTML "<table id='t02'>
  <tr>
    <th>File name</th>
    <th>Paired or single data</th>
    <th>File size</th>
    <th>File format</th>
  </tr>
  ";


if ($datatypeusefilelist eq "MiSeq"){ 
                chomp $pwd;   
                chdir("$dirresults/DATA/");        
                $j=1; 
                my @filesName = glob("$dirresults/DATA/*1*.gz");
                foreach my $file (@filesName) {
                    $nom          = infoFiles($file);
                    $nom =~s/$dirresults\/DATA\///;
                    #$pairedsingle = pairedFiles($file);
                    $pairedsingle = pairedFiles();
                    $seq = seqCount($file);
                    $ATGC = seqCountBases($file);
                    $sizecount    = infoSize($file);
                    $fastqformat  = checkFatsqFile($file);

                       print HTML "
                                  <tr>
                                  <td>$nom</td>
                                  <td>$pairedsingle</td> 
                                  <td>$seq sequences, $ATGC bases (size $sizecount octets)</td>
                                  <td>$fastqformat</td>	
                                  </tr>
                                  ";
                $j++;
                }#fin du foreach
                chdir($pwd);
}else{
                #Loader and run spades on ion torrent data (proton)
                my @files = glob("$dirresults/DATA/*.f*");
                foreach my $file (@files) {
                       $nom          = infoFiles($file);
                       $nom =~s/$dirresults\/DATA\///;
                       $pairedsingle = pairedFiles();
                       $seq = seqCount($file);
                       $ATGC = seqCountBases($file);
                       $sizecount    = infoSize($file);
                       $fastqformat  = checkFatsqFile($file);
                        print HTML "
                                  <tr>
                                  <td>$nom</td>
                                  <td>$pairedsingle</td> 
                                  <td>$seq sequences, $ATGC bases (size $sizecount octets)</td>
                                  <td>$fastqformat</td>	
                                  </tr>
                                 ";
            }   
}

print HTML "</table>
           ";
      

print HTML "<table id='t01'>
  <tr>
    <th>Sample number</th>
    <th>Sample name</th>
    <th>Sequence of assembled contigs</th>
    <th>Length and depth of assembled contigs</th>
    <th>Vector position in contigs</th>
    <th>Contigs with vector masked/cleaned</th>
    <th>Selected contigs (on length and depth)</th>
  </tr>
";

#[root@vm-galaxy-preprod sm_FOSMIDES]# more  /etc/httpd/conf.d/ZZZ_galaxy.conf 
#For HTML reports
#RewriteRule ^/galaxy/download/(.*) /galaxydata/galaxy-preprod/my_files/$1 [L]   --> Remplacement de my_files par my_workspace


if ($datatypeusefilelist eq "MiSeq"){ 
chomp $pwd;   
chdir("$dirresults/DATA/"); 
            $j=1; 
            
            my @filesName = glob("$dirresults/DATA/*1*.gz");
                       foreach my $file (@filesName) {
                            my $sample_name = basename($file); 
                            $sample_name =~s/.gz//;
                            my $cmdVN = "(cd $dirresults/DATA/echantillon_$j/; mkdir VNfiles/; chmod 777 VNfiles/; mv *VN.fasta VNfiles/.; mv *VM.fasta VNfiles/.;  cd VNfiles/; tar cvf VNfiles.tar *V*.fasta;) >& ./VN.log 2>&1;";
                            system $cmdVN;
                            my $url="$dirresults/DATA/echantillon_$j/VNfiles/VNfiles.tar";
                            $url=~s/$dir/\/galaxy\/download\//; 
                            my $url_selected_fasta="$dirresults/DATA/echantillon_$j/selected.fasta";
                            $url_selected_fasta=~s/$dir/\/galaxy\/download\//;  
                            my $url_filtred_contigs_txt="$dirresults/DATA/echantillon_$j/filtred_contigs.txt";
                            $url_filtred_contigs_txt=~s/$dir/\/galaxy\/download\//; 
                            my $url_vectordes="$dirresults/DATA/echantillon_$j/VECTOR.log";
                            $url_vectordes=~s/$dir/\/galaxy\/download\//; 
                            my $url_stats="$dirresults/DATA/echantillon_$j/stats.txt";
                            $url_stats=~s/$dir/\/galaxy\/download\//; 
                            
   
   
   
     
                           #contigs fasta files from spades compressed in a tar repository
                            my $cmdspades="(cd $dirresults/DATA/echantillon_$j/; mkdir SPADES_fasta/; chmod 777 SPADES_fasta/;) >& ./spades1_fasta.log 2>&1;";
                            system $cmdspades;
                    
                           `ls $dirresults/DATA/echantillon_$j/K*/final_contigs.fasta > $dirresults/DATA/echantillon_$j/Kfiles.txt`;
                  
                           open (KFILES,"$dirresults/DATA/echantillon_$j/Kfiles.txt")or die "Impossible to open $dirresults/DATA/echantillon_$j/Kfiles.txt";
                           open (CPKFILES,">$dirresults/DATA/echantillon_$j/cKfiles.sh") or die "Impossible to open $dirresults/DATA/echantillon_$j/cKfiles.sh"; 
                            
                            my $nb="1";
                            while(defined($lin = <KFILES>)) {
                                chomp($lin);
                                my @lin=split(/ /,$lin);
                                my $nameKfile = $sample_name."_".$nb.".fasta";
                                print CPKFILES "cp ".$lin." $dirresults/DATA/echantillon_$j/SPADES_fasta/$nameKfile ;";
                                $nb ++;
                            } 
                            
                            my $url_spades="$dirresults/DATA/echantillon_$j/contigs.fasta";
                            $url_spades=~s/$dir/\/galaxy\/download\//; 
                            
                            my $nbSeq= `grep -c "^>" $dirresults/DATA/echantillon_$j/selected.fasta`;
                            
                             my $fileName1="$sample_name"."_selected.fasta";
                             my $fileName2="$sample_name"."_description.txt"; 
                             my $fileName3="Vector_"."$sample_name";
                             my $fileName4="$sample_name".".tar";
                             my $fileName5="$sample_name"."_assembled_contigs.fasta";
                 
                            print HTML "
                                    <tr>
                                        <td>Sample N° $j</td>
                                        <td>$sample_name</td> 
                                        <td><a href=\"$url_spades\"> See </a> or <a href=\"$url_spades\" download=\"$fileName5\" target=\"_blank\"> download </a></td>			
                                        <td><a href=\"$url_stats\"> See </a> or <a href=\"$url_stats\" download=\"$fileName2\" target=\"_blank\"> download </a></td>		
                                        <td><a href=\"$url_vectordes\"> See or <a href=\"$url_vectordes\" download=\"$fileName3\" target=\"_blank\" > download </a></td>		
                                        <td><a href=\"$url\">See </a> or <a href=\"$url\" download=\"$fileName4\"> download </a></td>	
                                        <td><a href=\"$url_selected_fasta\"> $nbSeq contigs finally kept </a> or <a href=\"$url_selected_fasta\" download=\"$fileName1\" target=\"_blank\"> download $sample_name selected fasta</a></td>
                                      </tr>
                                     ";
                            $j++;
       }#end of foreach
       chdir($pwd);
}else{
        #Loader and run spades on ion torrent data (proton)
         $j=1;
         my @files = glob("$dirresults/DATA/*.f*");
         foreach my $file (@files) {
                $file100X = basename($file); 
                $file100X =~s/.gz//;
                my $rep = $file100X; 
                $rep =~s/.fq//;
                $rep =~s/.fastq//;
                my $cmdVN = "(cd $dirresults/DATA/SPADES_$rep/; mkdir VNfiles/; chmod 777 VNfiles/; mv *VN.fasta VNfiles/.; mv *VM.fasta VNfiles/.;  cd VNfiles/; tar cvf VNfiles.tar *V*.fasta;) >& ./VN.log 2>&1;";
                system $cmdVN;
                my $url="$dirresults/DATA/SPADES_$rep/VNfiles/VNfiles.tar";
                $url=~s/$dir/\/galaxy\/download\//; 
                my $url_selected_fasta="$dirresults/DATA/SPADES_$rep/selected.fasta";
                $url_selected_fasta=~s/$dir/\/galaxy\/download\//; 
                my $url_filtred_contigs_txt="$dirresults/DATA/SPADES_$rep/filtred_contigs.txt";
                $url_filtred_contigs_txt=~s/$dir/\/galaxy\/download\//; 
                #my $url_vectordes="$dirresults/DATA/SPADES_$rep/VECTORdes.txt";
                my $url_vectordes="$dirresults/DATA/SPADES_$rep/VECTOR.log";
                $url_vectordes=~s/$dir/\/galaxy\/download\//; 
                my $url_stats="$dirresults/DATA/SPADES_$rep/stats.txt";
                $url_stats=~s/$dir/\/galaxy\/download\//;
                
                
                #contigs fasta files from spades compressed in a tar repository
                my $cmdspades="(cd $dirresults/DATA/SPADES_$rep/; mkdir SPADES_fasta/; chmod 777 SPADES_fasta/;) >& ./spades1_fasta.log 2>&1;";
                system $cmdspades;
                    
                `ls $dirresults/DATA/SPADES_$rep/K*/final_contigs.fasta > $dirresults/DATA/SPADES_$rep/Kfiles.txt`;
                  
                open (KFILES,"$dirresults/DATA/SPADES_$rep/Kfiles.txt")or die "Impossible to open $dirresults/DATA/SPADES_$rep/Kfiles.txt";
                open (CPKFILES,">$dirresults/DATA/SPADES_$rep/cKfiles.sh") or die "Impossible to open $dirresults/DATA/SPADES_$rep/cKfiles.sh"; 
                     
                 my $nb="1";            
                while(defined($lin = <KFILES>)) {
                                chomp($lin);
                                my @lin=split(/ /,$lin);
                                my $nameKfile = $file100X."_".$nb.".fasta";
                                print CPKFILES "cp ".$lin." $dirresults/DATA/SPADES_$rep/SPADES_fasta/$nameKfile ;";
                                $nb ++;
                } 
	    
        
                my $url_spades="$dirresults/DATA/SPADES_$rep/contigs.fasta";
                $url_spades=~s/$dir/\/galaxy\/download\//; 
                            
                            
                            
                            
                 my $nbSeq= `grep -c "^>" $dirresults/DATA/SPADES_$rep/selected.fasta`;
                 
                 my $fileName1="$file100X"."_selected.fasta";
                 my $fileName2="$file100X"."_description.txt"; 
                 my $fileName3="Vector_"."$file100X";
                 my $fileName4="$file100X".".tar";
                 my $fileName5="$file100X"."_assembled_contigs.fasta";
                       
                print HTML "
                        <tr>
                            <td>Sample N° $j</td>
                            <td>$file100X</td>
                            <td><a href=\"$url_spades\"> See </a> or <a href=\"$url_spades\" download=\"$fileName5\" target=\"_blank\"> download </a></td>	
                            <td><a href=\"$url_stats\"> See </a> or <a href=\"$url_stats\" download=\"$fileName2\" target=\"_blank\"> download </a></td>		
                            <td><a href=\"$url_vectordes\"> See or <a href=\"$url_vectordes\" download=\"$fileName3\" target=\"_blank\" > download</a></td>		
                            <td><a href=\"$url\">See </a> or <a href=\"$url\" download=\"$fileName4\">download </a></td>
                            <td><a href=\"$url_selected_fasta\"> $nbSeq contigs finally kept </a> or <a href=\"$url_selected_fasta\" download=\"$fileName1\" target=\"_blank\"> download $file100X</a></td>		
                          </tr>
                         ";
                 $j++;
                }   
}

print HTML "</table>
            </body>
            </html>
           ";



#http:\/\/genoweb.toulouse.inra.fr\/\~sigenae\/sarah\/fosmides\/MODULE1\/
                
#Create HTML graphic
#print HTML "<link rel=\"stylesheet\" href=\"https:\/\/cdn.datatables.net\/1.10.6\/css\/jquery.dataTables.css\"></link>";
#print HTML "<link rel=\"stylesheet\" href=\"$scripts_fosmides/js/1.10.6/css/jquery.dataTables.css\"></link>";
print HTML "<link rel=\"stylesheet\" href=\"http:\/\/genoweb.toulouse.inra.fr\/\~sigenae\/sarah\/fosmides\/MODULE1\/js\/1.10.6\/css\/jquery.dataTables.css\"></link>";
#print HTML "<link rel=\"stylesheet\" href=\"https:\/\/maxcdn.bootstrapcdn.com\/bootstrap\/3.3.4\/css\/bootstrap.min.css\"></link>";
#print HTML "<link rel=\"stylesheet\" href=\"$scripts_fosmides/js/bootstrap/3.3.4/css/bootstrap.min.css\"></link>";
print HTML "<link rel=\"stylesheet\" href=\"http:\/\/genoweb.toulouse.inra.fr\/\~sigenae\/sarah\/fosmides\/MODULE1\/js/bootstrap/3.3.4/css/bootstrap.min.css\"></link>";
#print HTML "<script type=\"text\/javascript\" src=\"http:\/\/code.jquery.com\/jquery-2.1.3.min.js\"></script>";
#print HTML "<script type=\"text\/javascript\" src=\"$scripts_fosmides/js/jquery-2.1.3.min.js\"></script>";
print HTML "<script type=\"text\/javascript\" src=\"http:\/\/genoweb.toulouse.inra.fr\/\~sigenae\/sarah\/fosmides\/MODULE1\/js/jquery-2.1.3.min.js\"></script>";
#print HTML "<script type=\"text\/javascript\" src=\"http:\/\/code.highcharts.com\/4.1.4\/highcharts.js\"></script>";
#print HTML "<script type=\"text\/javascript\" src=\"$scripts_fosmides/js/4.1.4/highcharts.js\"></script>";
print HTML "<script type=\"text\/javascript\" src=\"http:\/\/genoweb.toulouse.inra.fr\/\~sigenae\/sarah\/fosmides\/MODULE1\/js/4.1.4/highcharts.js\"></script>";
#print HTML "<script type=\"text\/javascript\" src=\"https:\/\/cdn.datatables.net\/1.10.6\/js\/jquery.dataTables.min.js\"></script>";
#print HTML "<script type=\"text\/javascript\" src=\"$scripts_fosmides/js/1.10.6/js/jquery.dataTables.min.js\"></script>";
print HTML "<script type=\"text\/javascript\" src=\"http:\/\/genoweb.toulouse.inra.fr\/\~sigenae\/sarah\/fosmides\/MODULE1\/js/1.10.6/js/jquery.dataTables.min.js\"></script>";
#print HTML "<script src=\"https:\/\/code.highcharts.com\/modules\/exporting.js\"></script>";
#print HTML "<script src=\"$scripts_fosmides/js/modules/exporting.js\"></script>";
print HTML "<script src=\"http:\/\/genoweb.toulouse.inra.fr\/\~sigenae\/sarah\/fosmides\/MODULE1\/js/modules/exporting.js\"></script>";

#Generate un random number to change color for each sample
my $colorsRandom; 
sub colors {
         # Generate random numbers between 1 and 255
         my $r = int( rand(255)) + 1;
         my $g = int( rand(255)) + 1;
         my $b = int( rand(255)) + 1;
         my $colorsRandom = "$r, $g, $b";
         return $colorsRandom;
}
sub returndata {
                my ($j) = @_;
                my @seq=();
                @{$seq[$j]}=();
                if ($datatypeusefilelist eq "MiSeq"){ 
                    open (ALLCONT,"$dirresults/DATA/echantillon_$j/filtred_contigs.txt")or die "Impossible to open $dirresults/DATA/echantillon_$j/filtred_contigs.txt";                
                }else{
                    open (ALLCONT,"$dirresults/DATA/filtred_contigs.txt")or die "Impossible to open $dirresults/DATA/filtred_contigs.txt";	           
                }
	             while(defined($lin = <ALLCONT>)) {
		                            chomp($lin);
                                    my ($d) = $lin =~ /cov_(\d+\.*\d*)/;
                                    my ($l) = $lin =~ /length_(\d+)_cov/;
                                    push @{$seq[$j]}, "[$d, $l]";
                         } 
                my $data = join(',',@{$seq[$j]}) ;
                return $data;        
}                         
   

# generate data content for hightcharts
sub dataGraph {
        my ($file, $j) = @_;
        my $color=colors();
        my $serie;
        my $element;
        my $sample_name = basename($file); 
        $sample_name =~s/.gz//;
        my $valeurs=returndata($j);                 
        $serie = sprintf "{
                          name: '%s',
                          color: 'rgba($color, .5)', 
                          data: [$valeurs]
                          }",$sample_name; 
        return $serie;
}  
    

#Generate highcharts
sub highchart {
              my @filesName = @_;
              my $j=1;
              my $serie;
              my $element;
              foreach my $file (@filesName) {
                       $serie = dataGraph($file, $j);
                       $element .= ",".$serie;
                       $j ++;
              }
              #delete first character
              $element=~s/^.//;  
              
              #print HTML "<script src=\"https:\/\/code.highcharts.com\/highcharts.js\"></script>";
              #print HTML "<script src=\"$scripts_fosmides/js/highcharts.js\"></script>";
              #http:\/\/genoweb.toulouse.inra.fr\/\~sigenae\/sarah\/fosmides\/MODULE1\/
              print HTML "<script src=\"http:\/\/genoweb.toulouse.inra.fr\/\~sigenae\/sarah\/fosmides\/MODULE1\/js/highcharts.js\"></script>";
              #print HTML "<script src=\"https:\/\/code.highcharts.com\/modules\/exporting.js\"></script>";
              #print HTML "<script src=\"$scripts_fosmides/js/modules/exporting.js\"></script>";
              print HTML "<script src=\"http:\/\/genoweb.toulouse.inra.fr\/\~sigenae\/sarah\/fosmides\/MODULE1\/js/modules/exporting.js\"></script>";
              
              print HTML "<div id=\"container\" style=\"width:100%; height:400px;\"></div>";
              
              #freeze autoscale Y axe
              print HTML "<input id=\"min\" type=\"text\" value=\"0\"></input>";
              print HTML "<input id=\"max\" type=\"text\" value=\"200\"></input>";
              print HTML "<button id=\"change\">change Y axe</button></br>";


              #freeze autoscale X axe
              print HTML "<input id=\"minX\" type=\"text\" value=\"0\"></input>";
              print HTML "<input id=\"maxX\" type=\"text\" value=\"100\"></input>";
              print HTML "<button id=\"changeX\">change X axe</button>";



              #Ajout button add serie
              print HTML "<script>
                            \$(function () {
                                \$('#container').highcharts({
                                    chart: {
                                        type: 'scatter',
                                        zoomType: 'xy'
                                    },
                                    title: {
                                        text: 'Lenght and coverage'
                                    },
                                    subtitle: {
                                        text: 'for each sample'
                                    },
                                    xAxis: {
                                        title: {
                                            enabled: true,
                                            text: 'Coverage'
                                        },
                                        startOnTick: true,
                                        endOnTick: true,
                                        showLastLabel: true,
                                        id: 'my_x'
                                    },
                                    yAxis: {
                                        title: {
                                            text: 'Lenght'
                                        },
                                         id: 'my_y'
                                    },                                   
                                    plotOptions: {
                                        scatter: {
                                            marker: {
                                                radius: 5,
                                                states: {
                                                    hover: {
                                                        enabled: true,
                                                        lineColor: 'rgb(100,100,100)'
                                                    }
                                                }
                                            },
                                            states: {
                                                hover: {
                                                    marker: {
                                                        enabled: false
                                                    }
                                                }
                                            },
                                            tooltip: {
                                                headerFormat: '<b>{series.name}</b><br>',
                                                pointFormat: '{point.x} cov, {point.y} length'
                                            }
                                        }
                                    },
                                    series: [$element]
                                });
                                
                                
                                //http://jsfiddle.net/Wm7ft/
                                //freeze autoscale legend Y
                                // the button handlera
                                var chart = \$('#container').highcharts();
                                var yAxis = chart.get('my_y');
                                var extremes = yAxis.getExtremes();
                                \$('#min').attr('value', extremes.min);
                                \$('#max').attr('value', extremes.max);
                                \$('#change').click(function() {
                                   yAxis.setExtremes(\$('#min').val(),\$('#max').val());
                                    console.info(yAxis)
                                });
                                
                                //freeze autoscale legend X
                                // the button handlera
                                var chart = \$('#container').highcharts();
                                var xAxis = chart.get('my_x');
                                var extremesX = xAxis.getExtremes();
                                \$('#minX').attr('value', extremesX.min);
                                \$('#maxX').attr('value', extremesX.max);
                                \$('#changeX').click(function() {
                                    xAxis.setExtremes(\$('#minX').val(),\$('#maxX').val());
                                    console.info(xAxis)
                                });
                                                            
                                
                            });
                        </script>";  
                    
                    #Add button add serie
                       print HTML "<script>
                                  // tell the embed parent frame the height of the content
                                  if (window.parent && window.parent.parent){
                                    window.parent.parent.postMessage([\"resultsFrame\", {
                                      height: document.body.getBoundingClientRect().height,
                                      slug: \"None\"
                                    }], \"*\")
                                  }
                                  
                                  function hideChart() {
                                      if(Highcharts.charts.length<1) {
                                          return;
                                      }
                                      for(i=1;i<Highcharts.charts[0].series.length;i++){
                                          if(Highcharts.charts[0].series[i].visible==true) {
                                              Highcharts.charts[0].series[i].hide();
                                          }
                                          else {
                                              Highcharts.charts[0].series[i].show();
                                          }
                                      }
                                  }
                                </script>

                                <center><button type=\"button\" onclick=\"hideChart()\">Revert Hidden/Shown</button></center>";
                                
                                
}

if ($datatypeusefilelist eq "MiSeq"){highchart(glob("$dirresults/DATA/*2.f*q*gz"));}
else{highchart(glob("$dirresults/DATA/*.f*"));}

                              
#To open files in Galaxy, we have to parameter Apache like this :
# vi /etc/httpd/conf.d/galaxy.conf 
#To add this line: RewriteRule ^/galaxy/download/(.*) /work/galaxy/database/files/$1 [L]
#Then reload httpd :  /etc/init.d/httpd reload

if (! -e "results.html"){print STDERR "Pas de fichier html produit \n";}
else {my $cmdhtml = "(mv results.html $output) >& ./cp_result.log 2>&1";
system $cmdhtml;}


if ($datatypeusefilelist eq "MiSeq"){ 
chomp $pwd;   
chdir("$dirresults/DATA/"); 
            $j=1; 
            
            my @filesName = glob("$dirresults/DATA/*1*.gz");
                       foreach my $file (@filesName) {
                            my $sample_name = basename($file); 
                            $sample_name =~s/.gz//;
                            my $cmdzipmiseq1 ="(cp $dirresults/DATA/echantillon_$j/selected.fasta  $dirresults/DATA/$sample_name.fasta ;)>& ./cpmiseqzip.log 2>&1";
                            system $cmdzipmiseq1;
                            $j++;
       }#end of foreach
       chdir($pwd);
}else{
        #Loader and run spades on ion torrent data (proton)
         $j=1;
         my @files = glob("$dirresults/DATA/*.f*");
         foreach my $file (@files) {
                $file100X = basename($file); 
                $file100X =~s/.gz//;
                my $rep = $file100X; 
                $rep =~s/.fq//;
                $rep =~s/.fastq//;
                my $cmdzipproton1 ="(cp $dirresults/DATA/SPADES_$rep/selected.fasta  $dirresults/DATA/$rep.fasta ;)>& ./cpmiseqzip.log 2>&1";
                system $cmdzipproton1;
                 $j++;
                } 
                  
}



if ($datatypeusefilelist eq "MiSeq"){ 
        my $cmdzipmiseq ="(cp $dirresults/DATA/*.fasta .; zip $dirresults/DATA/miseq.zip *.fasta;)>& ./miseqzip.log 2>&1";
        system $cmdzipmiseq;
        my $zipmiseq="$dirresults/DATA/miseq.zip";
        if (! -e $zipmiseq){print STDERR "Pas de fichier ZIP produit \n";}
        else {my $cmdmiseqout = "(mv $dirresults/DATA/miseq.zip $outputzip) >& ./cp_miseqzip.log 2>&1";
        system $cmdmiseqout;}
     
     }
else{          
        my $cmdzip ="(cp $dirresults/DATA/*.fasta .; zip $dirresults/DATA/proton.zip *.fasta;)>& ./protonzip.log 2>&1";
        system $cmdzip;
        my $zipproton = "$dirresults/DATA/proton.zip";
        if (! -e $zipproton ){print STDERR "Pas de fichier ZIP produit \n";}
        else {my $cmdmiseqout = "(mv $dirresults/DATA/proton.zip $outputzip) >& ./cp_protonzip.log 2>&1";
        system $cmdmiseqout;}
        
}


#Close opened files
close( HTML );
close (ALLCONT);				
close (AVFASTA);
